package geniusweb.boa.acceptancestrategy;

import geniusweb.actions.Action;
import geniusweb.boa.BoaParty;
import geniusweb.boa.BoaState;
import geniusweb.boa.biddingstrategy.BiddingStrategy;

/**
 * Determines in which states a bid is acceptable. Typically used as one of the
 * components of a {@link BoaParty}. The protocol to be supported is SAOP.
 * <p>
 * <em>MUST</em> have an empty constructor.
 * 
 * <h2>More info</h2> Baarslag T., Hindriks K.V., Hendrikx M., Dirkzwager A.,
 * Jonker C.M. Decoupling Negotiating Agents to Explore the Space of Negotiation
 * Strategies. Proceedings of The Fifth International Workshop on Agent-based
 * Complex Automated Negotiations (ACAN 2012), 2012.
 * ​https://homepages.cwi.nl/~baarslag/pub/Decoupling_Negotiating_Agents_to_Explore_the_Space_of_Negotiation_Strategies_ACAN_2012.pdf
 */
public interface AcceptanceStrategy {

	/**
	 * given a candidate action, update the action considering its fitness and
	 * adjust if Accept would be a better choice. This normally is used to "fix"
	 * the Offers coming from the {@link BiddingStrategy}.
	 * 
	 * <p>
	 * <b>Note</b> for implementors: Previous offers that were done can be
	 * extracted from the negoState. configuration parameters can be extracted
	 * from {@link BoaState#getSettings()}. Generally we recommend to write
	 * implementations such that they can also be hard-configured, for easier
	 * use of the components in hard-coded settings.
	 * 
	 * @param proposedAction an action that is proposed (usually by the
	 *                       {@link BiddingStrategy} as the next action.
	 * @param boaState       the current {@link BoaState}.
	 * 
	 * 
	 * @return a filtered version of the action. For example, the
	 *         AcceptanceStrategy may deem an Accept better than a given Offer.
	 */
	Action filter(Action proposedAction, BoaState boaState);
}
